package hu.bme.mit.project.resources;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Properties;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import books.BookDB;

public class CounterProjectBuilder extends IncrementalProjectBuilder {

	private static final String OUTPUT_FILENAME = "count.properties";
	final static String outputFile = BookProjectNature.OUTPUTDIR + "/" + OUTPUT_FILENAME;
	Properties props = new Properties();

	@Override
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor)
			throws CoreException {
		try {
			getProject().refreshLocal(IResource.DEPTH_INFINITE, monitor);

			IFile fileResource = getProject().getFile(outputFile);
			File file = fileResource.getLocation().toFile();
			props = new Properties();
			boolean newFile = true;
			if (fileResource.exists()) {
				props.load(new FileInputStream(file));
				newFile = false;
			}
			switch (kind) {
			case CLEAN_BUILD:
				fileResource.delete(IResource.KEEP_HISTORY, monitor);
				break;
			case INCREMENTAL_BUILD:
			case AUTO_BUILD:
				IResourceDelta delta = getDelta(getProject());
				if (!newFile && delta != null) {
					incrementalBuild(delta, monitor);
					break;
				}
			case FULL_BUILD:
				fullBuild(monitor);
				break;

			default:
				break;
			}
			props.store(new FileOutputStream(file), OUTPUT_FILENAME);
			fileResource.refreshLocal(IResource.DEPTH_INFINITE, monitor);
			fileResource.setDerived(true, monitor);
			return null;
		} catch (IOException e) {
			throw new CoreException(new Status(IStatus.ERROR,
					Activator.PLUGIN_ID, "Cannot count", e));
		}
	}

	private void incrementalBuild(IResourceDelta delta, IProgressMonitor monitor)
			throws CoreException {
		delta.accept(new IResourceDeltaVisitor() {

			@Override
			public boolean visit(IResourceDelta delta) throws CoreException {
				try {
					IResource res = delta.getResource();
					if (res instanceof IFile) {
						IFile file = (IFile) res;
						if (new Path(BookProjectNature.DATADIR).equals(file.getParent()
								.getProjectRelativePath())) {
							switch (delta.getKind()) {
							case IResourceDelta.ADDED:
							case IResourceDelta.CHANGED:
								System.out.println("Updated " + file.getName());
								buildFile(file);
								break;
							case IResourceDelta.REMOVED:
								System.out.println("Removed " + file.getName());
								props.remove(file.getName());
								break;
							default:
								break;
							}
						}
					}
					return true;
				} catch (IOException e) {
					throw new CoreException(new Status(IStatus.ERROR,
							Activator.PLUGIN_ID, "Builder error", e));
				}
			}
		});

	}

	private void fullBuild(IProgressMonitor monitor) throws CoreException,
			IOException {
		IProject project = getProject();
		IFolder folder = project.getFolder(BookProjectNature.DATADIR);
		for (IResource res : folder.members()) {
			if (res instanceof IFile) {
				IFile file = (IFile) res;
				if (file.getFileExtension().toLowerCase().equals("books")) {
					buildFile(file);
				}
			}
		}
	}

	public void buildFile(IFile file) throws IOException {
		System.out.println(file);
		String name = file.getName();
		if (props.containsKey(name))
			props.remove(name);
		ResourceSet set = new ResourceSetImpl();
		Resource resource = set.createResource(URI.createPlatformResourceURI(
				file.getFullPath().toString(), true));
		resource.load(null);
		//TODO error handling missing here
		BookDB books = (BookDB) resource.getContents().get(0);
		String count = String.format("%d authors, %d books", books.getAuthors()
				.size(), books.getBooks().size());
		props.put(name, count);

	}
}
